@@ -8,7 +8,7 @@
 package Image::ExifTool::XMP;
 
 use strict;
-use vars qw(%specialStruct %dateTimeInfo $xlatNamespace);
+use vars qw(%specialStruct %dateTimeInfo %stdXlatNS);
 
 use Image::ExifTool qw(:DataAccess :Utils);
 
@@ -45,6 +45,7 @@ my $rdfClose = "</rdf:RDF>\n";
 my $xmpClose = "</x:xmpmeta>\n";
 my $pktCloseW =  "<?xpacket end='w'?>"; # writable by default
 my $pktCloseR =  "<?xpacket end='r'?>";
+my $noPad;
 
 #------------------------------------------------------------------------------
 # Get XMP opening tag (and set x:xmptk appropriately)
@@ -56,7 +57,7 @@ sub XMPOpen($)
     my $nv = $$et{NEW_VALUE}{$Image::ExifTool::XMP::x{xmptk}};
     my $tk;
     if (defined $nv) {
-        $tk = $et->GetNewValues($nv);
+        $tk = $et->GetNewValue($nv);
         $et->VerboseValue(($tk ? '+' : '-') . ' XMP-x:XMPToolkit', $tk);
         ++$$et{CHANGED};
     } else {
@@ -87,6 +88,40 @@ sub ValidateXMP($;$)
     return 1;
 }
 
+#------------------------------------------------------------------------------
+# Validate XMP property
+# Inputs: 0) ExifTool ref, 1) validate hash ref
+# - issues warnings if problems detected
+sub ValidateProperty($$)
+{
+    my ($et, $propList) = @_;
+
+    if ($$et{XmpValidate} and @$propList > 2) {
+        if ($$propList[0] =~ /^x:x[ma]pmeta$/ and
+            $$propList[1] eq 'rdf:RDF' and
+            $$propList[2] =~ /rdf:Description( |$)/)
+        {
+            if (@$propList > 3) {
+                if ($$propList[-1] =~ /^rdf:(Bag|Seq|Alt)$/) {
+                    $et->Warn("Ignored empty $$propList[-1] list for $$propList[-2]", 1);
+                } else {
+                    my $xmpValidate = $$et{XmpValidate};
+                    my $path = join('/', @$propList[3..$#$propList]);
+                    if (defined $$xmpValidate{$path}) {
+                        $et->Warn("Duplicate XMP property: $path") if defined $$xmpValidate{$path};
+                    } else {
+                        $$xmpValidate{$path} = 1;
+                    }
+                }
+            }
+        } elsif ($$propList[0] ne 'rdf:RDF' or
+                 $$propList[1] !~ /rdf:Description( |$)/)
+        {
+            $et->Warn('Improperly enclosed XMP property: ' . join('/',@$propList));
+        }
+    }
+}
+
 #------------------------------------------------------------------------------
 # Check XMP date values for validity and format accordingly
 # Inputs: 1) date string
@@ -130,7 +165,10 @@ sub CheckXMP($$$)
         unless (ref $$valPtr) {
             ($$valPtr, $warn) = InflateStruct($valPtr);
             # expect a structure HASH ref or ARRAY of structures
-            ref $$valPtr or return 'Improperly formed structure';
+            unless (ref $$valPtr) {
+                $$valPtr eq '' and $$valPtr = { }, return undef; # allow empty structures
+                return 'Improperly formed structure';
+            }
         }
         if (ref $$valPtr eq 'ARRAY') {
             return 'Not a list tag' unless $$tagInfo{List};
@@ -180,7 +218,7 @@ sub CheckXMP($$$)
             ($format eq 'rational' and ($$valPtr eq 'inf' or
              $$valPtr eq 'undef' or Image::ExifTool::IsRational($$valPtr))))
         {
-            return 'Not a floating point number' 
+            return 'Not a floating point number';
         }
         if ($format eq 'rational') {
             $$valPtr = join('/', Image::ExifTool::Rationalize($$valPtr));
@@ -235,13 +273,25 @@ sub SetPropertyPath($$;$$$$)
     my ($tagTablePtr, $tagID, $parentID, $structPtr, $propList, $isType) = @_;
     my $table = $structPtr || $tagTablePtr;
     my $tagInfo = $$table{$tagID};
+    my $flatInfo;
 
-    return if ref($tagInfo) ne 'HASH' or $$tagInfo{PropertyPath};
+    return if ref($tagInfo) ne 'HASH'; # (shouldn't happen)
 
-    # don't override existing main table entry if already set by a Struct
     if ($structPtr) {
+        my $flatID = $parentID . ucfirst($tagID);
+        $flatInfo = $$tagTablePtr{$flatID};
+        if ($flatInfo) {
+            return if $$flatInfo{PropertyPath};
+        } else {
+            # flattened tag doesn't exist, so create it now
+            # (could happen if we were just writing a structure)
+            $flatInfo = { Name => ucfirst($flatID), Flat => 1 };
+            AddTagToTable($tagTablePtr, $flatID, $flatInfo);
+        }
         $isType = 1 if $$structPtr{TYPE};
     } else {
+        # don't override existing main table entry if already set by a Struct
+        return if $$tagInfo{PropertyPath};
         # use property path from original tagInfo if this is an alternate-language tag
         my $srcInfo = $$tagInfo{SrcTagInfo};
         $$tagInfo{PropertyPath} = GetPropertyPath($srcInfo) if $srcInfo;
@@ -251,6 +301,7 @@ sub SetPropertyPath($$;$$$$)
             SetPropertyPath($tagTablePtr, $$tagInfo{RootTagInfo}{TagID});
             return if $$tagInfo{PropertyPath};
             warn "Internal Error: Didn't set path from root for $tagID\n";
+            warn "(Is the Struct NAMESPACE defined?)\n";
         }
     }
     my $ns = $$tagInfo{Namespace} || $$table{NAMESPACE};
@@ -294,14 +345,7 @@ sub SetPropertyPath($$;$$$$)
     # if this was a structure field and not a normal tag,
     # we set PropertyPath in the corresponding flattened tag
     if ($structPtr) {
-        my $flatID = $parentID . ucfirst($tagID);
-        $tagInfo = $$tagTablePtr{$flatID};
-        # create flattened tag now if necessary
-        # (could happen if we were just writing a structure)
-        unless ($tagInfo) {
-            $tagInfo = { Name => ucfirst($flatID), Flat => 1 };
-            AddTagToTable($tagTablePtr, $flatID, $tagInfo);
-        }
+        $tagInfo = $flatInfo;
         # set StructType flag if any containing structure has a TYPE
         $$tagInfo{StructType} = 1 if $isType;
     }
@@ -560,10 +604,14 @@ sub LimitXMPSize($$$$$$)
     push @$startPt, length($$dataPt);  # add end offset to list
     my $newData = substr($$dataPt, 0, $$startPt[0]);
     my $guid = '0' x 32;
+    my $sp = $noPad ? '' : ' ';
     # write the required xmpNote:HasExtendedXMP property
-    $newData .= "\n <$rdfDesc rdf:about='$about'\n  xmlns:xmpNote='$nsURI{xmpNote}'>\n" .
-                  "  <xmpNote:HasExtendedXMP>$guid</xmpNote:HasExtendedXMP>\n" .
-                  " </$rdfDesc>\n";
+    $newData .= "\n$sp<$rdfDesc rdf:about='${about}'\n$sp${sp}xmlns:xmpNote='$nsURI{xmpNote}'";
+    if ($et->Options('XMPShorthand')) {
+        $newData .= "\n$sp${sp}xmpNote:HasExtendedXMP='${guid}'/>\n";
+    } else {
+        $newData .= ">\n$sp$sp<xmpNote:HasExtendedXMP>$guid</xmpNote:HasExtendedXMP>\n$sp</$rdfDesc>\n";
+    }
 
     my ($i, %descSize, $start);
     # calculate all description block sizes
@@ -593,13 +641,66 @@ sub LimitXMPSize($$$$$$)
     return (\$extData, $guid);  # return extended XMP and its GUID
 }
 
+#------------------------------------------------------------------------------
+# Close out bottom-level property
+# Inputs: 0) current property path list ref, 1) longhand properties at each resource
+#         level, 2) shorthand properties at each resource level, 3) resource flag for
+#         each property path level (set only if XMPShorthand is enabled)
+sub CloseProperty($$$$)
+{
+    my ($curPropList, $long, $short, $resFlag) = @_;
+
+    my $prop = pop @$curPropList;
+    $prop =~ s/ .*//;       # remove list index if it exists
+    my $pad = $noPad ? '' : ' ' x (scalar(@$curPropList) + 1);
+    if ($$resFlag[@$curPropList]) {
+        # close this XMP structure with possible shorthand properties
+        if (length $$short[-1]) {
+            if (length $$long[-1]) {
+                # require a new Description if both longhand and shorthand properties
+                $$long[-2] .= ">\n$pad<$rdfDesc";
+                $$short[-1] .= ">\n";
+                $$long[-1] .= "$pad</$rdfDesc>\n";
+            } else {
+                # simply close empty property if all shorthand
+                $$short[-1] .= "/>\n";
+            }
+        } else {
+            # use "parseType" instead of opening a new Description
+            $$long[-2] .= ' rdf:parseType="Resource"';
+            $$short[-1] = length $$long[-1] ? ">\n" : "/>\n";
+        }
+        $$long[-1] .= "$pad</$prop>\n" if length $$long[-1];
+        $$long[-2] .= $$short[-1] . $$long[-1];
+        pop @$short;
+        pop @$long;
+    } elsif (defined $$resFlag[@$curPropList]) {
+        # close this top level Description with possible shorthand values
+        if (length $$long[-1]) {
+            $$long[-2] .= $$short[-1] . ">\n" . $$long[-1] . "$pad</$prop>\n";
+        } else {
+            $$long[-2] .= $$short[-1] . "/>\n"; # empty element (ie. all shorthand)
+        }
+        $$short[-1] = $$long[-1] = '';
+    } else {
+        # close this property (no chance of shorthand)
+        $$long[-1] .= "$pad</$prop>\n";
+        unless (@$curPropList) {
+            # add properties now that this top-level Description is complete
+            $$long[-2] .= ">\n" . $$long[-1];
+            $$long[-1] = '';
+        }
+    }
+    $#$resFlag = $#$curPropList;    # remove expired resource flags
+}
+
 #------------------------------------------------------------------------------
 # Write XMP information
 # Inputs: 0) ExifTool object reference, 1) source dirInfo reference,
 #         2) [optional] tag table reference
 # Returns: with tag table: new XMP data (may be empty if no XMP data) or undef on error
 #          without tag table: 1 on success, 0 if not valid XMP file, -1 on write error
-# Notes: May set dirInfo InPlace flag to rewrite with specified DirLen
+# Notes: May set dirInfo InPlace flag to rewrite with specified DirLen (=2 to allow larger)
 #        May set dirInfo ReadOnly flag to write as read-only XMP ('r' mode and no padding)
 #        May set dirInfo Compact flag to force compact (drops 2kB of padding)
 #        May set dirInfo MaxDataLen to limit output data length -- this causes ExtendedXMP
@@ -609,14 +710,16 @@ sub WriteXMP($$;$)
     my ($et, $dirInfo, $tagTablePtr) = @_;
     $et or return 1;    # allow dummy access to autoload this package
     my $dataPt = $$dirInfo{DataPt};
-    my (%capture, %nsUsed, $xmpErr, $tagInfo, $about);
+    my (%capture, %nsUsed, $xmpErr, $about);
     my $changed = 0;
     my $xmpFile = (not $tagTablePtr);   # this is an XMP data file if no $tagTablePtr
     # prefer XMP over other metadata formats in some types of files
     my $preferred = $xmpFile || ($$et{PreferredGroup} and $$et{PreferredGroup} eq 'XMP');
     my $verbose = $et->Options('Verbose');
+    my $compact = $et->Options('Compact') || 0;
     my $dirLen = $$dirInfo{DirLen};
     $dirLen = length($$dataPt) if not defined $dirLen and $dataPt;
+    $noPad = ($compact > 1);
 #
 # extract existing XMP information into %capture hash
 #
@@ -633,9 +736,14 @@ sub WriteXMP($$;$)
     delete $$et{XMP_IS_XML};
     delete $$et{XMP_IS_SVG};
 
+    # get value for new rdf:about
+    my $tagInfo = $Image::ExifTool::XMP::rdf{about};
+    if (defined $$et{NEW_VALUE}{$tagInfo}) {
+        $about = $et->GetNewValue($$et{NEW_VALUE}{$tagInfo}) || '';
+    }
+
     if ($xmpFile or $dirLen) {
         delete $$et{XMP_ERROR};
-        delete $$et{XMP_ABOUT};
         # extract all existing XMP information (to the XMP_CAPTURE hash)
         my $success = ProcessXMP($et, $dirInfo, $tagTablePtr);
         # don't continue if there is nothing to parse or if we had a parsing error
@@ -654,27 +762,35 @@ sub WriteXMP($$;$)
                     }
                 }
             } else {
+                $success = 2 if $success and $success eq '1';
                 if ($et->Warn($err, $success)) {
                     delete $$et{XMP_CAPTURE};
                     return undef;
                 }
             }
         }
-        $tagInfo = $Image::ExifTool::XMP::rdf{about};
-        if (defined $$et{NEW_VALUE}{$tagInfo}) {
-            $about = $et->GetNewValues($$et{NEW_VALUE}{$tagInfo}) || '';
+        if (defined $about) {
             if ($verbose > 1) {
-                my $wasAbout = $$et{XMP_ABOUT};
+                my $wasAbout = $$et{XmpAbout};
                 $et->VerboseValue('- XMP-rdf:About', UnescapeXML($wasAbout)) if defined $wasAbout;
                 $et->VerboseValue('+ XMP-rdf:About', $about);
             }
             $about = EscapeXML($about); # must escape for XML
             ++$changed;
         } else {
-            $about = $$et{XMP_ABOUT} || '';
+            $about = $$et{XmpAbout} || '';
         }
         delete $$et{XMP_ERROR};
-        delete $$et{XMP_ABOUT};
+
+        # call InitWriteDirs to initialize FORCE_WRITE flags if necessary
+        $et->InitWriteDirs({}, 'XMP') if $xmpFile and $et->GetNewValue('ForceWrite');
+        # set changed if we are ForceWrite tag was set to "XMP"
+        ++$changed if $$et{FORCE_WRITE}{XMP};
+
+    } elsif (defined $about) {
+        $et->VerboseValue('+ XMP-rdf:About', $about);
+        $about = EscapeXML($about); # must escape for XML
+        # (don't increment $changed here because we need another tag to be written)
     } else {
         $about = '';
     }
@@ -685,7 +801,7 @@ sub WriteXMP($$;$)
         $tagInfo = $Image::ExifTool::Extra{XMP};
         if ($tagInfo and $$et{NEW_VALUE}{$tagInfo}) {
             my $rtnVal = 1;
-            my $newVal = $et->GetNewValues($$et{NEW_VALUE}{$tagInfo});
+            my $newVal = $et->GetNewValue($$et{NEW_VALUE}{$tagInfo});
             if (defined $newVal and length $newVal) {
                 $et->VPrint(0, "  Writing XMP as a block\n");
                 ++$$et{CHANGED};
@@ -708,7 +824,7 @@ sub WriteXMP($$;$)
             my @propList = split('/',$path); # get property list
             my ($tag, $ns) = GetXMPTagID(\@propList);
             # translate namespace if necessary
-            $ns = $$xlatNamespace{$ns} if $$xlatNamespace{$ns};
+            $ns = $stdXlatNS{$ns} if $stdXlatNS{$ns};
             my ($grp, @g);
             # no "XMP-" added to most groups in exiftool RDF/XML output file
             if ($nsUsed{$ns} and (@g = ($nsUsed{$ns} =~ m{^http://ns.exiftool.ca/(.*?)/(.*?)/}))) {
@@ -772,7 +888,7 @@ sub WriteXMP($$;$)
         # to the ones used in this file
         $path = ConformPathToNamespace($et, $path);
         # find existing property
-        my $cap = $capture{$path}; 
+        my $cap = $capture{$path};
         # MicrosoftPhoto screws up the case of some tags, and some other software,
         # including Adobe software, has been known to write the wrong list type or
         # not properly enclose properties in a list, so we check for this
@@ -802,7 +918,7 @@ sub WriteXMP($$;$)
                 my $regex = quotemeta($fixPath);
                 $regex =~ s/ \d+/ \\d\+/g;  # match any list index
                 my $ok = $regex;
-                my ($ok2, $match, $i, @fixed, %fixed, $fixed, $changed);
+                my ($ok2, $match, $i, @fixed, %fixed, $fixed);
                 # check for incorrect list types
                 if ($regex =~ s{\\/rdf\\:(Bag|Seq|Alt)\\/}{/rdf:(Bag|Seq|Alt)/}g) {
                     # also look for missing bottom-level list
@@ -849,6 +965,7 @@ sub WriteXMP($$;$)
                     $et->Warn("Incorrect $wrn for existing $tg (not changed)");
                 } else {
                     # fix the incorrect property paths for all values of this tag
+                    my $didFix;
                     foreach $fixed (@fixed) {
                         my $match = shift @matches;
                         next if $fixed eq $match;
@@ -856,10 +973,13 @@ sub WriteXMP($$;$)
                         delete $capture{$match};
                         # remove xml:lang attribute from incorrect lang-alt list if necessary
                         delete $capture{$fixed}[1]{'xml:lang'} if $ok2 and $match !~ /^$ok2$/;
-                        $changed = 1;
+                        $didFix = 1;
                     }
                     $cap = $capture{$path} || $capture{$fixed[0]} unless @fixInfo;
-                    $et->Warn("Fixed incorrect $wrn for $tg", 1) if $changed;
+                    if ($didFix) {
+                        $et->Warn("Fixed incorrect $wrn for $tg", 1);
+                        ++$changed;
+                    }
                 }
             }
             last;
@@ -1019,7 +1139,7 @@ sub WriteXMP($$;$)
             (not $cap and $isCreating);
 
         # get list of new values (all done if no new values specified)
-        my @newValues = $et->GetNewValues($nvHash) or next;
+        my @newValues = $et->GetNewValue($nvHash) or next;
 
         # set language attribute for lang-alt lists
         $attrs{'xml:lang'} = $$tagInfo{LangCode} || 'x-default' if $writable eq 'lang-alt';
@@ -1095,6 +1215,7 @@ sub WriteXMP($$;$)
     my $maxDataLen = $$dirInfo{MaxDataLen};
     # get DataPt again because it may have been set by ProcessXMP
     $dataPt = $$dirInfo{DataPt};
+
     # return now if we didn't change anything
     unless ($changed or ($maxDataLen and $dataPt and defined $$dataPt and
         length($$dataPt) > $maxDataLen))
@@ -1107,15 +1228,17 @@ sub WriteXMP($$;$)
 # write out the new XMP information (serialize it)
 #
     # start writing the XMP data
-    my $newData = '';
+    my $useShorthand = $et->Options('XMPShorthand');
+    my (@long, @short, @resFlag);
+    $long[0] = $long[1] = $short[0] = '';
     if ($$et{XMP_NO_XPACKET}) {
         # write BOM if flag is set
-        $newData .= "\xef\xbb\xbf" if $$et{XMP_NO_XPACKET} == 2;
+        $long[-2] .= "\xef\xbb\xbf" if $$et{XMP_NO_XPACKET} == 2;
     } else {
-        $newData .= $pktOpen;
+        $long[-2] .= $pktOpen;
     }
-    $newData .= $xmlOpen if $$et{XMP_IS_XML};
-    $newData .= $xmpOpen . $rdfOpen;
+    $long[-2] .= $xmlOpen if $$et{XMP_IS_XML};
+    $long[-2] .= $xmpOpen . $rdfOpen;
 
     # initialize current property path list
     my (@curPropList, @writeLast, @descStart, $extStart);
@@ -1155,8 +1278,7 @@ sub WriteXMP($$;$)
             last unless @writeLast;
             @pathList = @writeLast;
             undef @writeLast;
-            $extStart = length $newData;
-            $newDesc = 1;   # start with a new description
+            $newDesc = 2;   # start with a new description for the extended data
         }
         $path = shift @pathList;
         my @propList = split('/',$path); # get property list
@@ -1203,82 +1325,103 @@ sub WriteXMP($$;$)
             }
         }
         # close out properties down to the common base path
-        while (@curPropList > $closeTo) {
-            ($prop = pop @curPropList) =~ s/ .*//;
-            $newData .= (' ' x scalar(@curPropList)) . " </$prop>\n";
-        }
+        CloseProperty(\@curPropList, \@long, \@short, \@resFlag) while @curPropList > $closeTo;
+
+        # open new description if necessary
         if ($newDesc) {
+            $extStart = length($long[-2]) if $newDesc == 2; # extended data starts after this
             # save rdf:Description start positions so we can reorder them if necessary
-            push @descStart, length($newData) if $maxDataLen;
+            push @descStart, length($long[-2]) if $maxDataLen;
             # open the new description
             $prop = $rdfDesc;
             %nsCur = %nsNew;            # save current namespaces
-            $newData .= "\n <$prop rdf:about='$about'";
+            my $sp = $noPad ? '' : ' ';
             my @ns = sort keys %nsCur;
+            $long[-2] .= "\n$sp<$prop rdf:about='${about}'";
             # generate et:toolkit attribute if this is an exiftool RDF/XML output file
             if (@ns and $nsCur{$ns[0]} =~ m{^http://ns.exiftool.ca/}) {
-                $newData .= "\n  xmlns:et='http://ns.exiftool.ca/1.0/'" .
+                $long[-2] .= "\n$sp${sp}xmlns:et='http://ns.exiftool.ca/1.0/'" .
                             " et:toolkit='Image::ExifTool $Image::ExifTool::VERSION'";
             }
-            foreach (@ns) {
-                $newData .= "\n  xmlns:$_='$nsCur{$_}'";
-            }
-            $newData .= ">\n";
+            $long[-2] .= "\n$sp${sp}xmlns:$_='$nsCur{$_}'" foreach @ns;
             push @curPropList, $prop;
+            # set resFlag to 0 to indicate base description when XMPShorthand enabled
+            $resFlag[0] = 0 if $useShorthand;
         }
-        # loop over all values for this new property
         my ($val, $attrs) = @{$capture{$path}};
         $debug and print "$path = $val\n";
-        # open new properties
-        my $attr;
+        # open new properties if necessary
+        my ($attr, $dummy);
         for ($n=@curPropList; $n<$#propList; ++$n) {
             $prop = $propList[$n];
             push @curPropList, $prop;
-            # remove list index if it exists
-            $prop =~ s/ .*//;
-            $attr = '';
+            $prop =~ s/ .*//;       # remove list index if it exists
+            # (we may add parseType and shorthand properties later,
+            #  so leave off the trailing ">" for now)
+            $long[-1] .= ($noPad ? '' : ' ' x scalar(@curPropList)) . "<$prop";
             if ($prop ne $rdfDesc and ($propList[$n+1] !~ /^rdf:/ or
                 ($propList[$n+1] eq 'rdf:type' and $n+1 == $#propList)))
             {
-                # need parseType='Resource' to avoid new 'rdf:Description'
-                $attr = " rdf:parseType='Resource'";
+                # check for empty structure
+                if ($propList[$n+1] =~ /:~dummy~$/) {
+                    $long[-1] .= " rdf:parseType='Resource'/>\n";
+                    pop @curPropList;
+                    $dummy = 1;
+                    last;
+                }
+                if ($useShorthand) {
+                    $resFlag[$#curPropList] = 1;
+                    push @long, '';
+                    push @short, '';
+                } else {
+                    # use rdf:parseType='Resource' to avoid new 'rdf:Description'
+                    $long[-1] .= " rdf:parseType='Resource'>\n";
+                }
+            } else {
+                $long[-1] .= ">\n"; # (will be no shorthand properties)
             }
-            $newData .= (' ' x scalar(@curPropList)) . "<$prop$attr>\n";
         }
         my $prop2 = pop @propList;  # get new property name
-        $prop2 =~ s/ .*//;          # remove list index if it exists
-        $newData .= (' ' x scalar(@curPropList)) . " <$prop2";
-        # write out attributes
-        foreach $attr (sort keys %$attrs) {
-            my $attrVal = $$attrs{$attr};
-            my $quot = ($attrVal =~ /'/) ? '"' : "'";
-            $newData .= " $attr=$quot$attrVal$quot";
+        # add element unless it was a dummy structure field
+        unless ($dummy or ($val eq '' and $prop2 =~ /:~dummy~$/)) {
+            $prop2 =~ s/ .*//;      # remove list index if it exists
+            my $pad = $noPad ? '' : ' ' x (scalar(@curPropList) + 1);
+            # (can't write as shortcut if it has attributes or CDATA)
+            if (defined $resFlag[$#curPropList] and not %$attrs and $val !~ /<!\[CDATA\[/) {
+                $short[-1] .= "\n$pad$prop2='${val}'";
+            } else {
+                $long[-1] .= "$pad<$prop2";
+                # write out attributes
+                foreach $attr (sort keys %$attrs) {
+                    my $attrVal = $$attrs{$attr};
+                    my $quot = ($attrVal =~ /'/) ? '"' : "'";
+                    $long[-1] .= " $attr=$quot$attrVal$quot";
+                }
+                $long[-1] .= length $val ? ">$val</$prop2>\n" : "/>\n";
+            }
         }
-        $newData .= length $val ? ">$val</$prop2>\n" : "/>\n";
-    }
-    # close off any open elements
-    while ($prop = pop @curPropList) {
-        $prop =~ s/ .*//;   # remove list index if it exists
-        $newData .= (' ' x scalar(@curPropList)) . " </$prop>\n";
     }
+    # close out all open properties
+    CloseProperty(\@curPropList, \@long, \@short, \@resFlag) while @curPropList;
+
     # limit XMP length and re-arrange if necessary to fit inside specified size
-    my $compact = $$dirInfo{Compact} || $et->Options('Compact');
     if ($maxDataLen) {
         # adjust maxDataLen to allow room for closing elements
         $maxDataLen -= length($rdfClose) + length($xmpClose) + length($pktCloseW);
-        $extStart or $extStart = length $newData;
-        my @rtn = LimitXMPSize($et, \$newData, $maxDataLen, $about, \@descStart, $extStart);
+        $extStart or $extStart = length $long[-2];
+        my @rtn = LimitXMPSize($et, \$long[-2], $maxDataLen, $about, \@descStart, $extStart);
         # return extended XMP information in $dirInfo
         $$dirInfo{ExtendedXMP} = $rtn[0];
         $$dirInfo{ExtendedGUID} = $rtn[1];
         # compact if necessary to fit
-        $compact = 1 if length($newData) + 101 * $numPadLines > $maxDataLen;
+        $compact = 1 if length($long[-2]) + 101 * $numPadLines > $maxDataLen;
     }
+    $compact = 1 if $$dirInfo{Compact};
 #
 # close out the XMP, clean up, and return our data
 #
-    $newData .= $rdfClose;
-    $newData .= $xmpClose unless $$et{XMP_NO_XMPMETA};
+    $long[-2] .= $rdfClose;
+    $long[-2] .= $xmpClose unless $$et{XMP_NO_XMPMETA};
 
     # remove the ExifTool members we created
     delete $$et{XMP_CAPTURE};
@@ -1289,27 +1432,30 @@ sub WriteXMP($$;$)
     # packet trailer, with a newline every 100 characters)
     unless ($$et{XMP_NO_XPACKET}) {
         my $pad = (' ' x 100) . "\n";
-        if ($$dirInfo{InPlace}) {
+        # get current XMP length without padding
+        my $len = length($long[-2]) + length($pktCloseW);
+        if ($$dirInfo{InPlace} and not ($$dirInfo{InPlace} == 2 and $len > $dirLen)) {
             # pad to specified DirLen
-            my $len = length($newData) + length($pktCloseW);
             if ($len > $dirLen) {
-                $et->Warn('Not enough room to edit XMP in place');
+                my $str = 'Not enough room to edit XMP in place';
+                $str .= '. Try XMPShorthand option' unless $$et{OPTIONS}{XMPShorthand};
+                $et->Warn($str);
                 return undef;
             }
             my $num = int(($dirLen - $len) / length($pad));
             if ($num) {
-                $newData .= $pad x $num;
+                $long[-2] .= $pad x $num;
                 $len += length($pad) * $num;
             }
-            $len < $dirLen and $newData .= (' ' x ($dirLen - $len - 1)) . "\n";
+            $len < $dirLen and $long[-2] .= (' ' x ($dirLen - $len - 1)) . "\n";
         } elsif (not $compact and not $xmpFile and not $$dirInfo{ReadOnly}) {
-            $newData .= $pad x $numPadLines;
+            $long[-2] .= $pad x $numPadLines;
         }
-        $newData .= ($$dirInfo{ReadOnly} ? $pktCloseR : $pktCloseW);
+        $long[-2] .= ($$dirInfo{ReadOnly} ? $pktCloseR : $pktCloseW);
     }
     # return empty data if no properties exist and this is allowed
     unless (%capture or $xmpFile or $$dirInfo{InPlace} or $$dirInfo{NoDelete}) {
-        $newData = '';
+        $long[-2] = '';
     }
     if ($xmpErr) {
         if ($xmpFile) {
@@ -1320,9 +1466,9 @@ sub WriteXMP($$;$)
         return undef;
     }
     $$et{CHANGED} += $changed;
-    $debug > 1 and $newData and print $newData,"\n";
-    return $newData unless $xmpFile;
-    Write($$dirInfo{OutFile}, $newData) or return -1;
+    $debug > 1 and $long[-2] and print $long[-2],"\n";
+    return $long[-2] unless $xmpFile;
+    Write($$dirInfo{OutFile}, $long[-2]) or return -1;
     return 1;
 }
 
@@ -1345,7 +1491,7 @@ This file contains routines to write XMP metadata.
 
 =head1 AUTHOR
 
-Copyright 2003-2015, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2018, Phil Harvey (phil at owl.phy.queensu.ca)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
